@@ -1,5 +1,5 @@
 /* source: sysutils.h */
-/* Copyright Gerhard Rieger */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __sysutils_h_included
@@ -27,7 +27,10 @@ union sockaddr_union {
 #if WITH_IP6
    struct sockaddr_in6 ip6;
 #endif /* WITH_IP6 */
-#if WITH_INTERFACE
+#if WITH_VSOCK
+   struct sockaddr_vm vm;
+#endif /* WITH_IP6 */
+#if _WITH_INTERFACE
    struct sockaddr_ll ll;
 #endif
 } ;
@@ -40,6 +43,10 @@ struct xiorange {
 } ;
 #endif /* _WITH_SOCKET */
 
+#if _WITH_INTERFACE
+extern const int one;
+#endif
+
 extern ssize_t writefull(int fd, const void *buff, size_t bytes);
 
 #if _WITH_SOCKET
@@ -68,10 +75,18 @@ extern char *sockaddr_inet4_info(const struct sockaddr_in *sa, char *buff, size_
 #if WITH_IP6
 extern char *sockaddr_inet6_info(const struct sockaddr_in6 *sa, char *buff, size_t blen);
 #endif /* WITH_IP6 */
+#if WITH_VSOCK
+extern char *sockaddr_vm_info(const struct sockaddr_vm *sa, char *buff, size_t blen);
+extern int sockaddr_vm_parse(struct sockaddr_vm *sa, const char *cid_str,
+                             const char *port_str);
+#endif /* WITH_VSOCK */
 #if !HAVE_INET_NTOP
 extern const char *inet_ntop(int pf, const void *binaddr,
 			     char *addrtext, socklen_t textlen);
 #endif
+extern int check_ip4addr(const char *address);
+extern int check_ip6addr(const char *address);
+extern int check_ipaddr(const char *address);
 
 #if defined(HAVE_SETGRENT) && defined(HAVE_GETGRENT) && defined(HAVE_ENDGRENT)
 extern int getusergroups(const char *user, gid_t *list, int *ngroups);
@@ -88,13 +103,20 @@ extern int parseport(const char *portname, int proto);
 extern int ifindexbyname(const char *ifname, int anysock);
 extern int ifindex(const char *ifname, unsigned int *ifindex, int anysock);
 
-extern int xiosetenv(const char *varname, const char *value, int overwrite);
+extern int xiosetenv(const char *varname, const char *value, int overwrite, const char *sep);
 extern int
 xiosetenv2(const char *varname, const char *varname2, const char *value,
-	   int overwrite);
+	   int overwrite, const char *sep);
+extern int
+xiosetenv3(const char *varname, const char *varname2, const char *varname3,
+	   const char *value, int overwrite, const char *sep);
 extern int xiosetenvulong(const char *varname, unsigned long value,
 			  int overwrite);
 extern int xiosetenvushort(const char *varname, unsigned short value,
 			   int overwrite);
+extern unsigned long int Strtoul(const char *nptr, char **endptr, int base, const char *txt);
+extern long long int Strtoll(const char *nptr, char **endptr, int base, const char *txt);
+extern double Strtod(const char *nptr, char **endptr, const char *txt);
+extern int xio_opensnifffile(const char *a, struct timeval *tv);
 
 #endif /* !defined(__sysutils_h_included) */
